unit Unit1;

{$MODE objfpc}{$H+}

interface

uses
  Classes, SysUtils, LResources, Forms, Controls, Graphics, Dialogs, ExtCtrls,
  Buttons, ExtDlgs, StdCtrls, LCLintf;

type

  TGraphicTypes = (gtRect, gtEllipse, gtImage, gtSelect);

  { TPaintObject }

  TPaintObject = class(TPersistent)

  private
    fBitmap: TBitmap;
    fGraphicType: TGraphicTypes;
    fHeight: integer;
    fWidth: integer;
    fX: integer;
    fY: integer;
  public
    constructor Create;
    destructor Destroy; override;
    property X: integer read fX write fX;
    property Y: integer read fY write fY;
    property Width: integer read fWidth write fWidth;
    property Height: integer read fHeight write fHeight;
    property Bitmap: TBitmap read fBitmap write fBitmap;
    property GraphicType: TGraphicTypes read fGraphicType write fGraphicType;
  end;


  { TPaintControl }

  TPaintControl = class(TCustomControl)

  private
    fObjList: TList;
    fCacheBuffer: TBitmap;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure MouseMove(Shift: TShiftState; X, Y: Integer); override;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure FillCacheBuffer;
    procedure Paint; override;
    property ObjList: TList read fObjList write fObjList;
    property CacheBuffer: TBitmap read fCacheBuffer write fCacheBuffer;
  end;


  { TForm1 }

  TForm1 = class(TForm)
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    Button7: TButton;
    ColorDialog1: TColorDialog;
    Label1: TLabel;
    OpenPictureDialog1: TOpenPictureDialog;
    Panel1: TPanel;
    Panel2: TPanel;
    Shape1: TShape;
    Shape2: TShape;
    procedure Button1Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button4Click(Sender: TObject);
    procedure Button5Click(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure Button7Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormResize(Sender: TObject);
    procedure FormShow(Sender: TObject);

  private
    { private declarations }
    PainterControl: TPaintControl;
  public
    { public declarations }
  end;

var
  Form1: TForm1;
  mx, my: integer;
  Buffer: TBitmap;
  Gt: TGraphicTypes;
  BGColor: TColor;
  FGColor: TColor;
  SelectedIndex: integer;

implementation

{ TForm1 }




procedure TForm1.FormCreate(Sender: TObject);
begin
  PainterControl := TPaintControl.Create(Self);
  PainterControl.Align := alClient;
  PainterControl.Parent := Panel2;
  PainterControl.DoubleBuffered := True;
  Buffer := TBitmap.create;
  mx := 200;
  my := 200;
end;

procedure TForm1.Button1Click(Sender: TObject);
begin
  GT := gtRect;
  Label1.Caption := 'Rectangle';
end;

procedure TForm1.Button2Click(Sender: TObject);
begin
  GT := gtEllipse;
  Label1.Caption := 'Ellipse';
end;

procedure TForm1.Button3Click(Sender: TObject);
begin
  Gt := gtImage;
  Label1.Caption := 'Image';
end;

procedure TForm1.Button4Click(Sender: TObject);
begin
  if ColorDialog1.Execute then
  begin
    BGColor := ColorDialog1.Color;
    Shape1.Brush.Color := BGColor;
  end;
end;

procedure TForm1.Button5Click(Sender: TObject);
begin
  if ColorDialog1.Execute then
  begin
    FGColor := ColorDialog1.Color;
    Shape2.Brush.Color := FGColor;
  end;
end;

procedure TForm1.Button6Click(Sender: TObject);
begin
  if PainterControl.ObjList.Count > 0 then
  begin
    PainterControl.ObjList.Delete(PainterControl.ObjList.Count - 1);
    SelectedIndex:=-1;
    PainterControl.FillCacheBuffer;
    PainterControl.Invalidate;
  end;
end;

procedure TForm1.Button7Click(Sender: TObject);
begin
  gt := gtSelect;
  Label1.Caption := 'Select';
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
  PainterControl.Free;
  Buffer.free;
end;

procedure TForm1.FormResize(Sender: TObject);
begin
  if Assigned(Buffer) then
  begin
    Buffer.Width := Width;
    Buffer.Height := Height;
  end;
  if Assigned(PainterControl.CacheBuffer) then
  begin
    PainterControl.CacheBuffer.Width := Width;
    PainterControl.CacheBuffer.Height := Height;
    PainterControl.FillCacheBuffer;
  end;
end;

procedure TForm1.FormShow(Sender: TObject);
begin
  PainterControl.FillCacheBuffer;
  BGColor := Clred;
  FGColor := ClYellow;
  Shape1.Brush.Color := BGColor;
  Shape2.Brush.Color := FGColor;
end;

procedure TPaintControl.FillCacheBuffer;
var i: integer;
begin
  fCacheBuffer.Canvas.Brush.Color := clWhite;
  fCacheBuffer.Canvas.FillRect(ClientRect);
  for i := 0 to ObjList.count - 1 do
    with TPaintObject(ObjList[i]) do
      fCacheBuffer.Canvas.Draw(X, Y, Bitmap);
end;


{ TPaintControl }


constructor TPaintControl.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  fObjList := TList.create;
  fCacheBuffer := TBitmap.Create;
end;

destructor TPaintControl.Destroy;
begin
  fObjList.free;
  fCacheBuffer.free;
  inherited Destroy;
end;

procedure TPaintControl.MouseMove(Shift: TShiftState; X, Y: Integer);
var Obj: TPaintObject;
begin
  if ssLeft in Shift then begin
    inherited MouseMove(Shift, X, Y);
    mx := x; my := y;
    if gt <> gtSelect then
    begin

      if Obj.GraphicType <> gtImage then
      begin
        Obj := TPaintObject(ObjList[ObjList.Count - 1]);
        Obj.Bitmap.Width := Abs(X - Obj.X + 1);
        Obj.Bitmap.Height := Abs(Y - Obj.Y + 1);
      end;


      case Obj.GraphicType of
        gtEllipse:
          begin
            Obj.Bitmap.Canvas.Brush.Color := clwhite;
            Obj.Bitmap.Canvas.FillRect(0, 0, X - Obj.X + 1, Y - Obj.Y + 1);
            Obj.Bitmap.Canvas.Brush.Color := BGColor;
            Obj.Bitmap.Canvas.Pen.Color := FGColor;
            Obj.Bitmap.Canvas.Ellipse(0, 0, X - Obj.X, Y - Obj.Y);
            Obj.Width := X - Obj.X;
            Obj.Height := Y - Obj.Y;
          end;
        gtRect:
          begin
            Obj.Bitmap.Canvas.Brush.Color := BGColor;
            Obj.Bitmap.Canvas.Pen.Color := FGColor;
            Obj.Bitmap.Canvas.FillRect(0, 0, X - Obj.X + 1, Y - Obj.Y + 1);
            Obj.Width := X - Obj.X;
            Obj.Height := Y - Obj.Y;
          end;
      end;
      Invalidate;
    end;
  end;
end;

procedure TPaintControl.MouseDown(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var Obj: TPaintObject;
  i: integer;
begin
  inherited MouseDown(Button, Shift, X, Y);
  if gt <> gtSelect then
  begin
    Obj := TPaintObject.create;
    Obj.X := X;
    Obj.Y := Y;
    Obj.GraphicType := Gt;
    if GT = gtImage then
    begin
      if Form1.OpenPictureDialog1.Execute then
        Obj.Bitmap.LoadFromFile(Form1.OpenPictureDialog1.FileName);
      Obj.Width := Obj.Bitmap.Width;
      Obj.Height := Obj.Bitmap.Height;
    end;
    ObjList.Add(Obj);
  end else
  begin
    for i := ObjList.Count - 1 downto 0 do
    begin
      Obj := TPaintObject(ObjList[i]);

      if PtInRect(Rect(Obj.x, Obj.y, Obj.x + Obj.width, Obj.y + Obj.height), Point(x, y)) then
      begin
        SelectedIndex := i;
        invalidate;
      end;
    end;
  end;
end;

procedure TPaintControl.MouseUp(Button: TMouseButton; Shift: TShiftState; X,
  Y: Integer);
var Obj: TPaintObject;
  Strm: TMemoryStream;
begin
  inherited MouseUp(Button, Shift, X, Y);
  if gt = gtEllipse then //folgendes ist nur, weil ich sonst die Transparenz nicht hinkriege.
  begin
    Obj := TPaintObject(ObjList[ObjList.Count - 1]);
    Strm := TMemoryStream.Create;
    Obj.Bitmap.SaveToStream(Strm);
    Strm.Position := 0;
    Obj.Bitmap.LoadFromStream(Strm);
    Strm.free;
    Obj.Bitmap.TransparentColor := Obj.Bitmap.Canvas.Pixels[0, 0];
    Obj.Bitmap.Transparent := True;
  end;
  if gt = gtImage then
  begin
    GT := gtRect;
    Form1.Label1.Caption := 'Rectangle';
  end;
  FillCacheBuffer;
  invalidate;
end;

procedure TPaintControl.Paint;
var i: integer;
var Obj: TPaintObject;
begin
  inherited Paint;
  with Buffer.Canvas do
  begin
    Draw(0, 0, CacheBuffer);
    TextOut(10, 10, inttostr(mx) + ' ' + inttostr(my));
    if not (GT = gtSelect) then
    begin
      if ObjList.Count > 0 then
        with TPaintObject(ObjList[ObjList.Count - 1]) do Draw(X, Y, Bitmap);
    end else
    begin
      if (ObjList.Count > 0) and (SelectedIndex>-1) then
      begin
       Obj := TPaintObject(ObjList[SelectedIndex]);
       DrawFocusRect(Rect(Obj.X - 1, Obj.Y - 1, Obj.X + Obj.Width + 1, Obj.Y + Obj.Height + 1));
      end;
    end;
    Canvas.Draw(0, 0, Buffer);
  end;
end;

{ TPaintObject }

constructor TPaintObject.Create;
begin
  fBitmap := TBitmap.Create;
end;

destructor TPaintObject.Destroy;
begin
  fBitmap.free;
  inherited Destroy;
end;

initialization
{$I unit1.lrs}

end.
